import 'package:flutter/material.dart';
import 'package:flutter_baidu_yingyan_trace/flutter_baidu_yingyan_trace.dart';
import 'package:flutter_datetime_picker/flutter_datetime_picker.dart';
import 'package:flutter_yingyan_trace_demo/customWidgets/border_line_button.dart';
import 'package:flutter_yingyan_trace_demo/customWidgets/map_appbar.dart';
import 'package:flutter_yingyan_trace_demo/constants.dart';
import 'package:flutter_yingyan_trace_demo/general/toast.dart';

/// 缓存页
class CachePage extends StatefulWidget {
  CachePage({Key key}) : super(key: key);

  @override
  _CachePageState createState() => _CachePageState();
}

class _CachePageState extends State<CachePage> {
  /// 结束时间
  DateTime _endTime;

  /// 开始时间
  DateTime _startTime;

  @override
  void initState() {
    _startTime = DateTime.now().subtract(Duration(days: 1));
    _endTime = DateTime.now();
    super.initState();
  }

  @override
  void dispose() {
    TraceController.shareInstance?.release();
    super.dispose();
  }

  /// 弹出时间选择框
  void _showTimePicker(BuildContext context, {bool isStart}) {
    DatePicker.showDateTimePicker(context,
        minTime: DateTime(2018),
        maxTime: DateTime.now(), onChanged: (DateTime date) {
      print('onChanged $date');
    }, onConfirm: (DateTime date) {
      print('onConfirm $date');
      setState(() {
        isStart ? _startTime = date : _endTime = date;
      });
    }, locale: LocaleType.zh);
  }

  /// 查询客户端缓存的轨迹信息
  void _queryTrackCacheInfo() async {
    /// 构建查询客户端缓存轨迹信息参数
    QueryTrackCacheInfoOption queryTrackCacheInfoOption =
        QueryTrackCacheInfoOption(
            tag: 1,
            serviceId: serviceID,
            entityNames: [entityName],
            startTime: _startTime.millisecondsSinceEpoch ~/ 1000,
            endTime: _endTime.millisecondsSinceEpoch ~/ 1000,
            needDistance: false,
            distanceFilter: 200);

    /// 发起请求
    TraceController.shareInstance.queryTrackCacheInfo(
        queryTrackCacheInfoOption: queryTrackCacheInfoOption,
        trackCallBack: TrackCallBack(
            onQueryTrackCacheInfoCallBack: (QueryTrackCacheInfoResult result) {
          print('-- 查询客户端缓存的轨迹信息回调 result = ${result?.toMap()}');
          showToast(msg: "${result?.toMap()}");
        }));

    // print('-- 查询客户端缓存的轨迹信息 flag = $flag');
  }

  /// 清空客户端缓存的轨迹信息
  void _clearTrackCache() async {
    /// 构建清空缓存参数
    ClearTrackCacheOption clearTrackCacheOption =
        ClearTrackCacheOption(tag: 1, serviceId: serviceID, trackCacheOptions: [
      TrackCacheOption(
          entityName: 'Modi',
          startTime: _startTime.millisecondsSinceEpoch ~/ 1000,
          endTime: _endTime.millisecondsSinceEpoch ~/ 1000,
          total: null),
      TrackCacheOption(
          entityName: 'Mark',
          startTime: _startTime.millisecondsSinceEpoch ~/ 1000,
          endTime: _endTime.millisecondsSinceEpoch ~/ 1000,
          total: null),
      TrackCacheOption(
          entityName: 'Jerry',
          startTime: _startTime.millisecondsSinceEpoch ~/ 1000,
          endTime: _endTime.millisecondsSinceEpoch ~/ 1000,
          total: null)
    ], entityNames: []);

    /// 清空缓存
    TraceController.shareInstance.clearTrackCache(
        clearTrackCacheOption: clearTrackCacheOption,
        trackCallBack: TrackCallBack(
            onClearTrackCacheCallBack: (ClearTrackCacheResult result) {
          print('--  清空客户端缓存的轨迹信息回调 result = ${result?.toMap()}');
          showToast(msg: "${result?.toMap()}");
        }));

    // print('--  清空客户端缓存的轨迹信息 flag = $flag');
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: BMFAppBar(
          title: "缓存管理",
          actions: [],
        ),
        body: Container(
            color: Colors.white,
            padding: EdgeInsets.all(15),
            child: Column(
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                Text('查询缓存轨迹的起始时间：'),
                Padding(padding: EdgeInsets.only(top: 10, bottom: 10)),
                Row(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    SizedBox(
                        width: 150,
                        height: 40,
                        child: BorderLineButton(
                          title: '开始：$_startTime',
                          textAlign: TextAlign.left,
                          onTap: () {
                            _showTimePicker(context, isStart: true);
                          },
                        )),
                    Padding(padding: EdgeInsets.only(left: 20)),
                    SizedBox(
                        width: 150,
                        height: 40,
                        child: BorderLineButton(
                          title: '结束：$_endTime',
                          textAlign: TextAlign.left,
                          onTap: () {
                            _showTimePicker(context, isStart: false);
                          },
                        )),
                  ],
                ),
                Padding(padding: EdgeInsets.only(top: 10, bottom: 10)),
                SizedBox(
                    width: 150,
                    child: BorderLineButton(
                      title: '查询缓存轨迹信息',
                      titleStyle: TextStyle(color: Colors.blue, fontSize: 14),
                      onTap: _queryTrackCacheInfo,
                    )),
                Padding(padding: EdgeInsets.only(top: 10, bottom: 10)),
                SizedBox(
                    width: 150,
                    child: BorderLineButton(
                      title: '清除缓存轨迹信息',
                      titleStyle: TextStyle(color: Colors.blue, fontSize: 14),
                      onTap: _clearTrackCache,
                    ))
              ],
            )));
  }
}
