import 'package:flutter/material.dart';
import 'package:flutter_baidu_yingyan_trace/flutter_baidu_yingyan_trace.dart';
import 'package:flutter_yingyan_trace_demo/customWidgets/map_appbar.dart';
import 'package:flutter_yingyan_trace_demo/customWidgets/setting_button.dart';
import 'package:flutter_yingyan_trace_demo/customWidgets/text_field.dart';
import 'package:flutter_yingyan_trace_demo/constants.dart';
import 'package:flutter_yingyan_trace_demo/general/toast.dart';

/// 创建entity页面
class AddEntityPage extends StatefulWidget {
  /// 添加entity成功回调
  final Future<void> Function(bool success) addEntityCallback;

  AddEntityPage({this.addEntityCallback, Key key}) : super(key: key);

  @override
  _AddEntityPageState createState() => _AddEntityPageState();
}

class _AddEntityPageState extends State<AddEntityPage> {
  final _entityNameController = TextEditingController();
  final _entityDescController = TextEditingController();

  /// 标记添加entity是否成功
  bool _isSuccess = false;

  @override
  void dispose() {
    TraceController.shareInstance?.release();
    super.dispose();
  }

  /// 创建entity
  void _addEntity() {
    if (_entityNameController.text == null ||
        _entityNameController.text == '') {
      showToast(msg: 'entityName不能为空', duration: 1);
      return;
    }

    /// 构建entity参数
    AddEntityOption addEntityOption = AddEntityOption(
        tag: 1,
        serviceId: serviceID,
        entityName: _entityNameController.text,
        entityDesc: _entityDescController.text != null
            ? _entityDescController.text
            : null);

    /// 发起创建entity请求
    TraceController.shareInstance.addEntity(
        addEntityOption: addEntityOption,
        entityCallBack:
            EntityCallBack(onAddEntityCallBack: (AddEntityResult result) {
          setState(() {
            _isSuccess = result?.status == 0 ? true : false;
          });
          showToast(msg: '${result.toMap()}', duration: 1);
          print('${result.toMap()}');
        }));
  }

  /// 返回上一界面
  Future<void> _goBack(BuildContext context) async {
    if (widget.addEntityCallback != null) {
      widget.addEntityCallback(_isSuccess);
    }
    Navigator.pop(context);
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: BMFAppBar(
          onBack: () {
            _goBack(context);
          },
          title: "新建一个终端实体",
          actions: [
            SettingButton(
              icon: 'images/general/icon_done.png',
              onTap: _addEntity,
            )
          ],
        ),
        backgroundColor: Colors.white,
        body: Container(
            child: Column(children: [
          Padding(padding: EdgeInsets.only(top: 10, bottom: 10)),
          Row(children: [
            SizedBox(
              height: 20,
              width: 20,
            ),
            BMFTextField(
                controller: _entityNameController,
                title: "entityName：",
                width: 300,
                placeholder: '字母数字汉字下划线的组合')
          ]),
          Padding(padding: EdgeInsets.only(top: 10, bottom: 10)),
          Row(children: [
            SizedBox(
              height: 20,
              width: 20,
            ),
            BMFTextField(
                controller: _entityDescController,
                title: "entityDesc  ：",
                width: 300,
                placeholder: '终端实体描述')
          ]),
        ])));
  }
}
