import 'package:flutter/material.dart';
import 'package:flutter_baidu_yingyan_trace/flutter_baidu_yingyan_trace.dart';
import 'package:flutter_yingyan_trace_demo/customWidgets/map_appbar.dart';
import 'package:flutter_yingyan_trace_demo/entityPage/add_entity_page.dart';
import 'package:flutter_yingyan_trace_demo/entityPage/entity_list_item.dart';
import 'package:flutter_yingyan_trace_demo/entityPage/entity_page.dart';
import 'package:flutter_yingyan_trace_demo/constants.dart';
import 'package:flutter_yingyan_trace_demo/general/toast.dart';

/// entity列表页
class EntityListPage extends StatefulWidget {
  EntityListPage({Key key}) : super(key: key);

  @override
  _EntityListPageState createState() => _EntityListPageState();
}

class _EntityListPageState extends State<EntityListPage> {
  /// entity列表
  List<EntityInfo> _entityList;

  @override
  void dispose() {
    _entityList?.clear();
    TraceController.shareInstance?.release();
    super.dispose();
  }

  @override
  initState() {
    super.initState();

    /// 查询entity列表
    _queryEntityList();
  }

  /// 查询终端entitu列表
  Future<void> _queryEntityList() async {
    /// 检索entity参数
    EntityListSearchOption entityListSearchOption = EntityListSearchOption(
        tag: 1, serviceId: serviceID, coordType: CoordType.BD09LL);

    TraceController.shareInstance.queryEntityList(
        entityListSearchOption: entityListSearchOption,
        entityCallBack: EntityCallBack(
            onEntitySearchListCallBack: (EntityListSearchResult result) {
          print('-- 查询entity list回调 result = ${result?.toMap()}');
          showToast(msg: '查询entity list ${result.message}');
          setState(() {
            _entityList = result.entities;
          });
        }));
  }

  /// 添加entity
  void _addEntity() {
    Navigator.push(
        context,
        MaterialPageRoute(
            builder: (context) => AddEntityPage(
                  addEntityCallback: (bool success) async {
                    print('$success');
                    if (success) {
                      /// 刷新列表
                      _queryEntityList();
                    }
                  },
                )));
  }

  /// 删除entity
  void _deleteEntity(BuildContext context, int index) {
    EntityInfo entity = _entityList[index];

    setState(() {
      /// 数组删除
      _entityList.removeAt(index);
      Scaffold.of(context)
          .showSnackBar(SnackBar(content: Text('${entity.entityName} delete')));
    });

    /// 服务端删除
    DeleteEntityOption deleteEntityOption = DeleteEntityOption(
        tag: 1, serviceId: serviceID, entityName: entity.entityName);
    TraceController.shareInstance.deleteEntity(
        deleteEntityOption: deleteEntityOption,
        entityCallBack:
            EntityCallBack(onDeleteEntityCallBack: (DeleteEntityResult result) {
          print('${result.message}');
        }));
  }

  /// 跳转entity页面
  void _goEntityPage(EntityInfo entityInfo) {
    if (entityInfo == null) {
      return;
    }
    Navigator.push(
        context,
        MaterialPageRoute(
            builder: (context) => EntityPage(entityInfo: entityInfo)));
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
        appBar: BMFAppBar(
          title: "Entity列表",
          actions: [
            IconButton(icon: Icon(Icons.add), onPressed: _addEntity),
          ],
        ),
        backgroundColor: Colors.white,
        body: RefreshIndicator(
            child: ListView.builder(
              itemBuilder: _itemBuilder,
              itemCount: _entityList == null ? 0 : _entityList.length,
            ),
            onRefresh: _queryEntityList));
  }

  /// 构建entity list item
  Widget _itemBuilder(BuildContext context, int index) {
    final EntityInfo entity = _entityList[index];
    return Dismissible(
      key: Key(entity.toString()),
      child: EntityListItem(
        entityInfo: entity,
        onTap: () {
          print('$index');
          _goEntityPage(entity);
        },
      ),
      onDismissed: (direction) async {
        /// 左右滑删除
        if (direction == DismissDirection.startToEnd ||
            direction == DismissDirection.endToStart) {
          _deleteEntity(context, index);
        }
      },
      background: Container(color: Colors.red),
    );
  }
}
