import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_baidu_yingyan_trace/flutter_baidu_yingyan_trace.dart';
import 'package:flutter_datetime_picker/flutter_datetime_picker.dart';
import 'package:flutter_yingyan_trace_demo/customWidgets/border_line_button.dart';
import 'package:flutter_yingyan_trace_demo/customWidgets/map_appbar.dart';
import 'package:flutter_yingyan_trace_demo/customWidgets/map_raised_button.dart';
import 'package:flutter_yingyan_trace_demo/customWidgets/selected_box.dart';
import 'package:flutter_yingyan_trace_demo/customWidgets/text_field.dart';
import 'package:flutter_yingyan_trace_demo/constants.dart';
import 'package:flutter_yingyan_trace_demo/general/themes.dart';
import 'package:intl/intl.dart';

typedef HistoryTrackSettingPageCallback = Function(
    QueryHistoryTrackOption option);

/// 历史轨迹查询设置
class HistoryTrackSettingPage extends StatefulWidget {
  final HistoryTrackSettingPageCallback callback;

  const HistoryTrackSettingPage({Key key, this.callback}) : super(key: key);

  @override
  _HistoryTrackSettingPageState createState() =>
      _HistoryTrackSettingPageState();
}

class _HistoryTrackSettingPageState extends State<HistoryTrackSettingPage> {
  /// 开始时间
  DateTime _startTime;

  /// 结束时间
  DateTime _endTime;

  /// 纠偏
  List<bool> _processBoxStatus = [true, false, false, false];

  /// 交通
  List<bool> _trafficBoxStatus = [true, false, false];

  /// 补偿里程
  List<bool> _supplementModelBoxStatus = [true, false, false, false, false];

  /// 排序
  List<bool> _sortBoxStatus = [true, false];

  /// 坐标类型
  List<bool> _coordTypeStatus = [false, true];

  /// 精度过滤
  final _accuracyController = TextEditingController(text: "100");

  /// 低速阈值
  final _thresholdController = TextEditingController(text: "20");

  /// 交通方式
  int _transportMode = 1;

  /// 里程补偿
  int _supplementMode = 0;

  /// 排序规则
  int _sortType = 0;

  /// 坐标类型
  int _coordType = 2;

  @override
  void initState() {
    _startTime = DateTime.now().subtract(Duration(days: 1));
    _endTime = DateTime.now();

    super.initState();
  }

  @override
  void dispose() {
    _accuracyController.dispose();
    _thresholdController.dispose();

    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: BMFAppBar(
        title: "查询设置",
      ),
      body: Container(
        color: Colors.white,
        padding: EdgeInsets.all(10),
        child: ListView(
          children: [
            _queryTimeWidget(),
            _processWidget(),
            _accuracyWidget(),
            _thresholdWidget(),
            _trafficWidget(),
            _supplementModelWidget(),
            _sortTypeWidget(),
            _coordTypeWidget(),
            _footerWidget(),
          ],
        ),
      ),
    );
  }

  /// 弹出时间选择框
  void _showTimePicker(BuildContext context, {bool isStart}) {
    DatePicker.showDateTimePicker(context,
        minTime: DateTime(2018),
        maxTime: DateTime.now(), onChanged: (DateTime date) {
      print('onChanged $date');
    }, onConfirm: (DateTime date) {
      print('onConfirm $date');
      setState(() {
        isStart ? _startTime = date : _endTime = date;
      });
    }, locale: LocaleType.zh);
  }

  /// 纠偏
  void _onTapProcessBox(int idx) {
    _processBoxStatus[idx] = !_processBoxStatus[idx];
    setState(() {});
  }

  /// 交通
  void _onTapTrafficBox(int idx) {
    for (int i = 0; i < _trafficBoxStatus.length; i++) {
      _trafficBoxStatus[i] = i == idx;
    }
    _transportMode = idx + 1;
    setState(() {});
  }

  /// 补偿里程
  void _onTapSupplementModelBox(int idx) {
    for (int i = 0; i < _supplementModelBoxStatus.length; i++) {
      _supplementModelBoxStatus[i] = i == idx;
    }
    _supplementMode = idx;
    setState(() {});
  }

  /// 排序
  void _onTapSortType(int idx) {
    for (int i = 0; i < _sortBoxStatus.length; i++) {
      _sortBoxStatus[i] = i == idx;
    }
    _sortType = idx;
    setState(() {});
  }

  /// 坐标类型
  void _onTapCoordType(int idx) {
    for (int i = 0; i < _coordTypeStatus.length; i++) {
      _coordTypeStatus[i] = i == idx;
    }
    _coordType = idx + 1;
    setState(() {});
  }

  /// 取消 确定
  void _onPressedFooter(int idx) {
    if (idx == 1) {
      QueryHistoryTrackOption queryHistoryTrackOption = QueryHistoryTrackOption(
        tag: 1,
        serviceId: serviceID,
        entityName: entityName,
        startTime: _startTime.millisecondsSinceEpoch ~/ 1000,
        endTime: _endTime.millisecondsSinceEpoch ~/ 1000,
        isProcessed: _processBoxStatus[0],
        processOption: QueryTrackProcessOption(
          denoise: _processBoxStatus[1],
          vacuate: _processBoxStatus[2],
          mapMatch: _processBoxStatus[3],
          radiusThreshold: int.parse(_accuracyController.text.isNotEmpty
              ? _accuracyController.text
              : "100"),
          transportMode: TrackProcessOptionTransportMode.values[_transportMode],
        ),
        supplementMode:
            TrackProcessOptionSupplementMode.values[_supplementMode],
        lowSpeedThreshold: double.parse(_thresholdController.text.isNotEmpty
            ? _thresholdController.text
            : "20"),
        sortType: TrackSortType.values[_sortType],
        outputCoordType: CoordType.values[_coordType],
        pageIndex: 1,
        pageSize: 5000,
      );

      print(queryHistoryTrackOption.toMap());

      if (widget.callback != null) {
        widget.callback(queryHistoryTrackOption);
      }
    }

    Navigator.pop(context);
  }

  /// 时间格式化 yyyy-MM-dd HH:mm
  String dateFormatToMinute(DateTime date) {
    return DateFormat("yyyy-MM-dd HH:mm").format(date);
  }

  /** ------------------------------------------------------ */

  /// 起止时间
  Widget _queryTimeWidget() {
    Size screenSize = MediaQuery.of(context).size;
    double timeWith = (screenSize.width - 30.0) / 2.0;

    return Container(
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Text(
            "起止时间：",
            style: settingTitleStyle,
          ),
          SizedBox(height: 10),
          Row(
            mainAxisAlignment: MainAxisAlignment.center,
            children: [
              SizedBox(
                width: timeWith,
                child: BorderLineButton(
                  title: "开始：${dateFormatToMinute(_startTime)}",
                  titleStyle: _timeStyle,
                  onTap: () => _showTimePicker(context, isStart: true),
                ),
              ),
              SizedBox(width: 10),
              SizedBox(
                width: timeWith,
                child: BorderLineButton(
                  title: "结束：${dateFormatToMinute(_endTime)}",
                  titleStyle: _timeStyle,
                  onTap: () => _showTimePicker(context, isStart: false),
                ),
              ),
            ],
          ),
          _separator(),
        ],
      ),
    );
  }

  /// 纠偏与数据分析
  Widget _processWidget() {
    return Container(
      padding: EdgeInsets.only(top: 10),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Text(
            "纠偏与数据分析：",
            style: settingTitleStyle,
          ),
          SizedBox(height: 10),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              SelectedBox(
                title: "纠偏",
                selected: _processBoxStatus[0],
                onTap: () => _onTapProcessBox(0),
              ),
              SelectedBox(
                title: "去噪",
                selected: _processBoxStatus[1],
                onTap: () => _onTapProcessBox(1),
              ),
              SelectedBox(
                title: "抽稀",
                selected: _processBoxStatus[2],
                onTap: () => _onTapProcessBox(2),
              ),
              SelectedBox(
                title: "绑路",
                selected: _processBoxStatus[3],
                onTap: () => _onTapProcessBox(3),
              ),
              SizedBox(width: 15),
            ],
          ),
          _separator(),
        ],
      ),
    );
  }

  /// 精度过滤
  Widget _accuracyWidget() {
    return Container(
      padding: EdgeInsets.only(top: 10),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Text(
            "精度过滤(单位：米)：",
            style: settingTitleStyle,
          ),
          SizedBox(height: 10),
          BMFTextField(
            controller: _accuracyController,
            title: "过滤值：",
            width: 150,
          ),
          _separator(),
        ],
      ),
    );
  }

  /// 阈值
  Widget _thresholdWidget() {
    return Container(
      padding: EdgeInsets.only(top: 10),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Text(
            "低速阈值需要在纠偏开启的情况下返回：",
            style: settingTitleStyle,
          ),
          SizedBox(height: 10),
          BMFTextField(
            controller: _thresholdController,
            title: "低速阈值：",
            width: 160,
          ),
          _separator(),
        ],
      ),
    );
  }

  /// 交通方式
  Widget _trafficWidget() {
    return Container(
      padding: EdgeInsets.only(top: 10),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Text(
            "交通方式：",
            style: settingTitleStyle,
          ),
          SizedBox(height: 10),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              SelectedBox(
                title: "驾车",
                selected: _trafficBoxStatus[0],
                onTap: () => _onTapTrafficBox(0),
              ),
              SelectedBox(
                title: "骑行",
                selected: _trafficBoxStatus[1],
                onTap: () => _onTapTrafficBox(1),
              ),
              SelectedBox(
                title: "步行",
                selected: _trafficBoxStatus[2],
                onTap: () => _onTapTrafficBox(2),
              ),
              SizedBox(width: 15),
            ],
          ),
          _separator(),
        ],
      ),
    );
  }

  /// 里程补偿方式
  Widget _supplementModelWidget() {
    return Container(
      padding: EdgeInsets.only(top: 10),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Text(
            "里程补偿方式：",
            style: settingTitleStyle,
          ),
          SizedBox(height: 10),
          SelectedBox(
            title: "不补充",
            selected: _supplementModelBoxStatus[0],
            onTap: () => _onTapSupplementModelBox(0),
          ),
          SizedBox(height: 5),
          SelectedBox(
            title: "直线距离",
            selected: _supplementModelBoxStatus[1],
            onTap: () => _onTapSupplementModelBox(1),
          ),
          SizedBox(height: 5),
          SelectedBox(
            title: "最短驾车路线",
            selected: _supplementModelBoxStatus[2],
            onTap: () => _onTapSupplementModelBox(2),
          ),
          SizedBox(height: 5),
          SelectedBox(
            title: "最短骑行路线",
            selected: _supplementModelBoxStatus[3],
            onTap: () => _onTapSupplementModelBox(3),
          ),
          SizedBox(height: 5),
          SelectedBox(
            title: "最短步行路线",
            selected: _supplementModelBoxStatus[4],
            onTap: () => _onTapSupplementModelBox(4),
          ),
          _separator(),
        ],
      ),
    );
  }

  /// 排序规则
  Widget _sortTypeWidget() {
    return Container(
      padding: EdgeInsets.only(top: 10),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Text(
            "排序规则：",
            style: settingTitleStyle,
          ),
          SizedBox(height: 10),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              SelectedBox(
                title: "升序",
                selected: _sortBoxStatus[0],
                onTap: () => _onTapSortType(0),
              ),
              SelectedBox(
                title: "降序",
                selected: _sortBoxStatus[1],
                onTap: () => _onTapSortType(1),
              ),
              SizedBox(width: 15),
            ],
          ),
          _separator(),
        ],
      ),
    );
  }

  /// 坐标类型
  Widget _coordTypeWidget() {
    return Container(
      padding: EdgeInsets.only(top: 10),
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Text(
            "坐标类型：",
            style: settingTitleStyle,
          ),
          SizedBox(height: 10),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            children: [
              SelectedBox(
                title: "百度经纬度坐标",
                selected: _coordTypeStatus[1],
                onTap: () => _onTapCoordType(1),
              ),
              SelectedBox(
                title: "国测局加密坐标",
                selected: _coordTypeStatus[0],
                onTap: () => _onTapCoordType(0),
              ),
              SizedBox(width: 15),
            ],
          ),
          _separator(),
        ],
      ),
    );
  }

  /// 取消 确定
  Widget _footerWidget() {
    return Container(
      color: Colors.white,
      padding: EdgeInsets.all(15),
      child: Row(
        mainAxisAlignment: MainAxisAlignment.spaceEvenly,
        children: [
          BMFRaisedButton(
            title: "取消",
            onPressed: (bool selected) => _onPressedFooter(0),
          ),
          BMFRaisedButton(
            title: "确定",
            onPressed: (bool selected) => _onPressedFooter(1),
          ),
        ],
      ),
    );
  }

  /// 分割线
  Widget _separator() {
    return Container(
      color: separateColor,
      height: 0.5,
      margin: EdgeInsets.only(top: 10),
    );
  }
}

final _timeStyle = TextStyle(
  color: Colors.blue,
  fontSize: 12,
);
